import calendar
import os
import re
from datetime import datetime
from functools import partial

from doku.client import Client
from doku.config_manager import ConfigManager


class Context:
    client: Client
    config: ConfigManager
    editor: str
    today: str
    diary_prefix = 'diary'
    __today = None

    def __init__(self):
        self.editor = os.environ.get('EDITOR', 'vim')
        self.config = ConfigManager()
        self.__new_client()
        self.__today = datetime.now()

    def __new_client(self):
        info = self.config['connection']
        self.client = Client(
            info['domain'],
            info['username'],
            info['password'],
            ssl=info.get('ssl', True),
            basepath=info.get('path', '/')
        )

    def indexpage(self):
        return self.config.get('settings', 'indexpage', 'index')

    def day_path(self, year, month, day):
        return ':{}:{:04d}:{:02d}:{:02d}'.format(
            self.diary_prefix, year, month, day)

    def month_path(self, year, month):
        return ':{}:{:04d}:{:02d}:{}'.format(
            self.diary_prefix, year, month, self.indexpage())

    def year_path(self, year):
        return ':{}:{:04d}:{}'.format(
            self.diary_prefix, year, self.indexpage())

    def diary_path(self):
        return ':{}:{}'.format(self.diary_prefix, self.indexpage())

    def today(self):
        _t = self.__today
        return self.day_path(_t.year, _t.month, _t.day)

    def this_month(self):
        _t = self.__today
        return self.month_path(_t.year, _t.month)

    def this_year(self):
        _t = self.__today
        return self.year_path(_t.year)

    def fill_month_view(self):
        content = self.client.call('wiki.getPage', self.this_month())
        if content != '':
            return

        print('Generate month view...')
        _t = self.__today
        days = calendar.monthrange(_t.year, _t.month + 1)[1]
        lines = []
        for d in range(1, days + 1, 7):
            link = partial(self.day_path, year=_t.year, month=_t.month)
            current = ['[[{}|{}]]'.format(link(day=d+i), d+i)
                       for i in range(0, 7) if d + i <= days]
            lines.append('|{}|'.format('|'.join(current)))

        lines.insert(0, _t.strftime('====== %B ======%n'))

        self.client.call(
            'wiki.putPage',
            self.this_month(),
            '\n'.join(lines),
            {'sum': 'Auto-generated with doku command line tool'}
        )

    def fill_year_view(self):
        content = self.client.call('wiki.getPage', self.this_year())
        if content != '':
            return

        print('Generate year view...')
        link = lambda x: '  * [[{}|{}]]'.format(
                    self.month_path(self.__today.year, x),
                    calendar.month_name[x]
               )
        lines = [link(m+1) for m in range(0, 12)]

        lines.insert(0, self.__today.strftime('====== %Y ======%n'))

        self.client.call(
            'wiki.putPage',
            self.this_year(),
            '\n'.join(lines),
            {'sum': 'Auto-generated with doku command line tool'}
        )

    def update_diary_root(self):
        content = self.client.call('wiki.getPage', self.diary_path())
        if self.year_path(self.__today.year) in content:
            return

        print('Regenerate diary root...')
        pages = self.client.call('dokuwiki.getPagelist', ':diary', [])
        lines = []
        for page in [p for p in pages
                     if re.match(
                         r'^diary:\d{4}:%s$' %
                         (self.indexpage()),
                         p['id'])
                     ]:
            lines.append('  * [[:{}]]'.format(page['id']))

        lines.insert(0, '====== Diary ======\n')

        self.client.call(
            'wiki.putPage',
            self.diary_path(),
            '\n'.join(lines),
            {'sum': 'Auto-generated with doku command line tool'}
        )




from configparser import ConfigParser
from pathlib import Path
import os

class ConfigManager:
    __config: ConfigParser

    def __init__(self):
        self.__config = None

    def __load(self):
        filepath = os.path.join(Path.home(), '.doku.ini')

        self.__config = ConfigParser()
        if not os.path.isfile(filepath):
            raise FileNotFoundError(f'File not found: {filepath}')
        self.__config.read(filepath)

    def __getitem__(self, key):
        if self.__config is None:
            self.__load()

        try:
            return self.__config[key]
        except KeyError:
            return None

    def get(self, section, key, default):
        try:
            return self.__config[section][key]
        except KeyError:
            return default

    def sections(self):
        if self.__config is None:
            self.__load()

        return self.__config.sections()



from xmlrpc.client import Transport


class CookiesTransport(Transport):
    """A Python3 xmlrpc.client.Transport subclass that retains cookies."""
    def __init__(self):
        super().__init__()
        self._cookies = dict()

    def send_headers(self, connection, headers):
        if self._cookies:
            cookies = map(lambda x: '='.join(x), self._cookies.items())
            connection.putheader("Cookie", "; ".join(cookies))
        super().send_headers(connection, headers)

    def parse_response(self, response):
        """parse and store cookie"""
        try:
            for header in response.msg.get_all("Set-Cookie"):
                cookie = header.split(";", 1)[0]
                cookieKey, cookieValue = cookie.split("=", 1)
                self._cookies[cookieKey] = cookieValue
        finally:
            return super().parse_response(response)




import click
import os
import re
import tempfile
from datetime import datetime
from subprocess import Popen
from textwrap import indent

from doku.context import Context

pass_ctx = click.make_pass_decorator(Context)

"""
Potential:
    - dokuwiki.appendPage -> Diary
"""

@click.group()
@click.pass_context
def cli(ctx):
    ctx.obj = Context()

@cli.command()
@click.option('--namespace', default='')
@pass_ctx
def list(ctx, namespace):
    pages = ctx.client.call('dokuwiki.getPagelist', namespace, [])
    for p in pages:
        t = datetime.utcfromtimestamp(p['rev']).strftime('%Y-%m-%d %H:%M:%S')
        print('[{date}] {page}'.format(page=p['id'], date=t))

@cli.command()
@click.argument('name')
@pass_ctx
def cat(ctx, name):
    print(ctx.client.call('wiki.getPage', name))

@cli.command()
@click.argument('query')
@pass_ctx
def search(ctx, query):
    results = ctx.client.call('dokuwiki.search', query)
    for hit in results:
        print('> {}'.format(hit['id']))
        if 'snippet' in hit:
            content = re.sub(
                r'</strong>|<strong class="search_hit">',
                '',
                hit['snippet']
            )
            print(indent(content, ' ' * 4, lambda line: True))
        print()

@cli.group()
def diary():
    pass

@diary.command()
@pass_ctx
def show(ctx):
    content = ctx.client.call('wiki.getPage', ctx.today())
    if content == '':
        print('No entries for today, yet!')
    else:
        print(content)

@diary.command()
@pass_ctx
def log(ctx):
    ctx.fill_month_view()
    ctx.fill_year_view()
    ctx.update_diary_root()

    fp, path = tempfile.mkstemp(prefix='doku', suffix='.wiki', text=True)
    os.close(fp)

    with Popen([ctx.editor, path]) as proc:
        proc.wait()

    with open(path, 'r') as fp:
        content = fp.read()
        if not all(l == '' for l in content.split('\n')[-2:]):
            content = '\n'.join([content, ''])

        content = '\n'.join([
            datetime.now().strftime('=== %r ===\n'),
            content
        ])

        ret = ctx.client.call(
            'dokuwiki.appendPage',
            ctx.today(),
            content,
            {'sum': 'Log added with doku command line tool'}
        )

        if ret:
            print('Document saved.')
        else:
            print('Something went wrong. :(')

    os.remove(path)

@cli.command()
@click.argument('name')
@pass_ctx
def edit(ctx, name):
    data = ctx.client.call('wiki.getPage', name)

    fp, path = tempfile.mkstemp(prefix='doku', suffix='.wiki', text=True)
    os.write(fp, data.encode())
    os.close(fp)

    with Popen([ctx.editor, path]) as proc:
        proc.wait()

    with open(path, 'r') as fp:
        ret = ctx.client.call(
            'wiki.putPage',
            name,
            fp.read(),
            {'sum': 'Updated with doku command line tool'}
        )

        if ret:
            print('Document saved.')
        else:
            print('Something went wrong. :(')

    os.remove(path)


if __name__ == '__main__':
    cli()




from setuptools import setup, find_packages
from doku import __version__


with open("README.rst", "r") as f:
    long_description = f.read()

setup(name='doku',
      version=__version__,
      packages=find_packages(),
      author="Balazs Nadasdi",
      author_email="balazs.nadasdi@cheppers.com",
      long_description=long_description,
      url="https://github.com/yitsushi/doku",
      zip_safe=True,
      include_package_data=True,
      install_requires=['click'],
      classifiers=[
          "Programming Language :: Python :: 3",
          "License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)",
          "Operating System :: OS Independent",
      ],
      entry_points="""
      [console_scripts]
      doku = doku.main:cli
      """)



import xmlrpc.client

from doku.cookie_safe_transport import CookiesSafeTransport
from doku.cookie_transport import CookiesTransport


class Client:
    __proxy: xmlrpc.client.ServerProxy

    def __init__(self, domain, username, password, ssl=True, basepath='/'):
        transport = CookiesSafeTransport() if ssl else CookiesTransport()
        proto = 'https' if ssl else 'http'

        self.__proxy = xmlrpc.client.ServerProxy(
            '{proto}://{domain}{path}/lib/exe/xmlrpc.php'.format(
                proto=proto,
                domain=domain,
                path=basepath
            ),
            transport=transport
        )
        if not self.__proxy.dokuwiki.login(username, password):
            raise 'Login Failed'

    def call(self, method, *args):
        return getattr(self.__proxy, method)(*args)

